/*
 * main.c
 *
 *  Christophe Lucas <c.lucas@ifrance.com>
 *
 *  This program is under GPL term :-)
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/ioctl.h>

#include <linux/compiler.h>
#include <linux/videodev.h>

#include <SDL/SDL.h>

extern void bayer2rgb24(unsigned char *dst, unsigned char *src, long int WIDTH, long int HEIGHT) ;

int main( int argc, char **argv)
{
    int			fd1, done ;
    unsigned long int	x, y, f;
    unsigned char	*s, *d;

    Uint32		rmask, gmask, bmask, amask;
    Uint32		delay = 20 ; /* 1/25 * 1000 */

    SDL_Surface		*screen;
    SDL_Surface		*show ;
    SDL_Rect		dest;
    SDL_Event		event;

    /* For debug */
    int nb_frame = 0 ; 
    time_t deb , end ;
    /* For debug */

    struct v4l2_cropcap cropcap;
    struct v4l2_crop crop;
    struct v4l2_format format;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif
    
    done = 0 ;

    if (argc != 5) {
        fprintf(stdout, "Usage: %s <input.bayer> <width> <height> <downscalefact>\n"
		"Where downscalefact must be 1,2 or 4 and width*downscalefact "
		"and height*downscalefact\nmust be multiple of 16 and <= than max res\n", 
		argv[0]);
	return 1;
    } else {
	fprintf( stdout , "\nPress ESC to stop playing...\n\n" );
    }

    x = atoi(argv[2]);
    y = atoi(argv[3]);
    f = atoi(argv[4]);

    if (SDL_Init (SDL_INIT_VIDEO) < 0) {
	fprintf (stderr, "Impossible d'initialiser SDL\n");
	exit (1);
    }
    

    screen = SDL_SetVideoMode( x, y, 24, SDL_SWSURFACE|SDL_DOUBLEBUF|SDL_ANYFORMAT);
    if( screen == NULL ) {
	fprintf (stderr, "Impossible de passer en %ldx%ld en %d bpp \n", x , y , 24 );
	exit (1);
    }

    SDL_WM_SetCaption( "Webcam", NULL );
	
    s = (char*)malloc(x * y);
    if( s == NULL ) {
	perror("Lack of memory !! :-(");
	exit(1);
    }

    d = (char*)malloc(x * y *3);
    if( d == NULL ) {
	perror("Lack of memory !! :-(");
	exit(1);
    }

    fd1 = open(argv[1], O_RDONLY);
    if( fd1 == -1 ) {
	perror("Error\n");
	exit(1);
    }

    cropcap.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if( ioctl(fd1,VIDIOC_CROPCAP,&cropcap) == -1 ) {
	fprintf(stderr, "VIDIOC_CROPCAP failed");
	exit(-1);
    }

    crop.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
    crop.c.top = 0;
    crop.c.left = 0;
    crop.c.width = x*f;
    crop.c.height = y*f;
    if( ioctl(fd1,VIDIOC_S_CROP,&crop) == -1 ) {
    	fprintf(stderr, "VIDIOC_S_CROP failed");
	exit(-1);
    }

    format.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
    format.fmt.pix.width = x;
    format.fmt.pix.height = y;
    format.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
    if( ioctl(fd1,VIDIOC_S_FMT,&format) == -1 ) {
    	fprintf(stderr, "VIDIOC_S_FMT failed");
	exit(-1);
    }



    time( &deb );

    while( !done ) {
	int ret ;

	ret = read(fd1, s, x * y);
	if( ret != (x * y ) ) {
	    printf("We will have fragmented picture :-(\n" );
	}
	nb_frame ++ ;
	bayer2rgb24(d, s, x, y);
	show = SDL_CreateRGBSurfaceFrom( (void *)d, x, y, 24, x*3, rmask, gmask, bmask, 0 );
	if( show == NULL ) {
	    printf("Erreur lors de la cration de la SDL_Surface\n");
	} else {
	    dest.x = 0 ; 
	    dest.y = 0 ;
	    dest.w = x ;
	    dest.h = y ;
    
	    if( SDL_BlitSurface( show, NULL, screen, &dest) != 0 ) {
		printf("SDL_BlitSurface failed.\n");
	    }
	    SDL_Flip( screen );
	}
	while ( SDL_PollEvent(&event) ) {
	    switch (event.type) {
		case SDL_KEYDOWN:
		    if ( event.key.keysym.sym == SDLK_ESCAPE ) {
			done = 1;
		    }
		    break;
		case SDL_QUIT:
		    done = 1;
		break;
		default:
		break;
	    }
	}   
	SDL_Delay( delay ) ;
    }
    time( &end );
    printf("I have been playing %d frame(s) in %lg second(s).\n" , nb_frame, difftime( end , deb ) );
    close(fd1);
    SDL_FreeSurface( screen );
    SDL_FreeSurface( show ) ;
    SDL_Quit();
return 0;
}
